# Copyright 2021 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from datetime import date

from dateutil.relativedelta import relativedelta

from odoo import api, fields, models


class AccOperation(models.Model):
    _name = "acc.operation"
    _inherit = ["acc.operation", "api.enedis"]

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    client_id = fields.Char("Client ID Enedis")
    secret_id = fields.Char("Secret ID Enedis")
    is_data_enedis = fields.Boolean("Data récupérés", default=False)

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------
    @api.model
    def _auto_get_enedis_data(self):
        """This method is called from a cron job.
        It is used to get data from Enedis with API.
        """
        day_birthday = date.today() - relativedelta(months=1, days=10)
        records = self.search(
            [
                ("birthday_date", "=", day_birthday.day),
                ("client_id", "!=", False),
                ("secret_id", "!=", False),
            ]
        )
        for rec in records:
            last_date = day_birthday + relativedelta(months=1, days=-1)
            token = self.access_token()
            rec.get_curves(day_birthday, last_date, token)

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def get_curves(self, date_start, date_end, token=None):
        # Ask token to API
        if not token:
            token = self.access_token()

        # Création du lot
        batch_name = (
            "Courbes du "
            + str(date_start)
            + " au "
            + str(date_end)
            + " - Opération :"
            + str(self.name)
        )
        batch = self.env["queue.job.batch"].get_new_batch(batch_name)
        # Load consommation data by PRM
        for delivery_counter_id in self.acc_delivery_ids:
            # Mise en file d'attente des appels API consommateur
            self.with_context(job_batch=batch).with_delay().definitive_load_curves(
                date_start, date_end, delivery_counter_id, token=token
            )

        # Load production data by PRM
        for injection_counter_id in self.acc_injection_ids:
            # Mise en file d'attente des appels API producteur
            self.with_context(job_batch=batch).with_delay().definitive_load_curves(
                date_start, date_end, injection_counter_id, token=token
            )
        # lancement de la file d'attente
        batch.enqueue()

    def get_curves_all(self, date_start):
        # Calcul du nombre de mois entre la date de début de contrat
        # de l'opération et la date du jour pour lancer la récupération des données
        num_months = (date.today().year - date_start.year) * 12 + (
            date.today().month - date_start.month
        )

        date_start_it = date_start
        date_end_it = date_start + relativedelta(months=1, days=-1)

        token = self.access_token()
        i = 1
        while i < num_months:
            self.get_curves(date_start_it, date_end_it, token)
            date_start_it = date_start_it + relativedelta(months=1)
            date_end_it = date_start_it + relativedelta(months=1, days=-1)
            i += 1

    def get_perimeter(self):
        for operation in self:
            operation.perimeter()

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
