# Copyright 2021 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import _, fields, models
from odoo.exceptions import UserError


class AccOperationWizard(models.TransientModel):
    _name = "acc.operation.wizard"
    _description = "Récupération des courbes pour une date donnée"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    date_start = fields.Date("Date de début")
    date_end = fields.Date("Date de fin")

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def get_curves(self):
        if not self.date_end and not self.date_start:
            raise UserError(
                _("Les champs Date de début et Date de fin sont obligatoires")
            )
        if (self.date_end - self.date_start).days > 31:
            raise UserError(_("L'intervalle de temps ne doit pas dépasser 31 Jours"))
        context = dict(self._context or {})
        if context.get("active_ids", False):
            self.env["acc.operation"].browse(context.get("active_ids")).get_curves(
                self.date_start, self.date_end
            )
        return {"type": "ir.actions.act_window_close"}

    def get_curves_all(self):
        context = dict(self._context or {})
        if context.get("active_ids", False):
            op = self.env["acc.operation"].browse(context.get("active_ids"))
            if not op.date_start_contract:
                raise UserError(
                    _(
                        "Renseigner une date de début de contrat pour pouvoir récupérer les courbes à partie de cette date."
                    )
                )
            date_start = op.date_start_contract
            self.env["acc.operation"].browse(context.get("active_ids")).get_curves_all(
                date_start
            )
        return {"type": "ir.actions.act_window_close"}

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
